package compiler;

import ast.Node;

/**
 * <p>Title: Programming mini project 2 Java-- compiler</p>
 * <p>Description: An entry into the symbol table to represent an anonymous scope, eg a compound
 * statement.<br>
 * Implements the factory pattern, use getInstance to get a new new ScopeEntry</p>
 * <p>Copyright: Copyright (c) 2004 Nicholas Cameron</p>
 * <p>Company: </p>
 * @author Nicholas Cameron
 * @version 1.0
 */

public class ScopeSymbol extends Symbol
{
    //symbol table to contain symbols inside the scope
    private SymbolTable sTable;

    //private constructor, name is a unique name for the scope generated by getInstance
    //st is a SymbolTable for the scope
    private ScopeSymbol(String name, SymbolTable st, Node decl)
    {
        super(name, "void", decl);
        sTable = st;
        sTable.setParentEntry(this);
    }

    //used by getInstance to create a unique name for each scope
    private static int nextScope = 0;

    /**
     * create a new instance of ScopeEntry
     * @param st a SymbolTable that contains symbols in the new scope
     * @param decl the node where the scope is stored
     * @return a new, and unique, ScopeEntry object
     */
    public static ScopeSymbol getInstance(SymbolTable st, Node decl)
    {
        return new ScopeSymbol("scope" + (++nextScope), st, decl);
    }

    public String getKind()
    {
        return "scope";
    }

    public SymbolTable getSymbolTable()
    {
        return sTable;
    }
}