package test;

import junit.framework.TestCase;
import compiler.*;

/**
 * <p>Title: Programming mini project 2 Java-- compiler</p>
 * <p>Description: Tests the symbolEquals method in the Symbol class and its sub-classes</p>
 * <p>Copyright: Copyright (c) 2004 Nicholas Cameron</p>
 * <p>Company: </p>
 * @author Nicholas Cameron
 * @version 1.0
 */

public class TestSymbolEquals extends TestCase
{
    public TestSymbolEquals()
    {
    }

    public void testTypeEqualsSelf()
    {
        TypeSymbol ts1 = new TypeSymbol("test1");

        assertTrue("TypeSymbols should be equal if testing self", ts1.symbolEquals(ts1));
    }

    public void testTypeEquals()
    {
        TypeSymbol ts1 = new TypeSymbol("test1");
        TypeSymbol ts2 = new TypeSymbol("test1");

        assertTrue("TypeSymbols should be equal", ts1.symbolEquals(ts2));
    }

    public void testTypeNotEquals()
    {
        TypeSymbol ts1 = new TypeSymbol("test1");
        TypeSymbol ts2 = new TypeSymbol("test2");

        assertFalse("Names are different --> not symbolsEquals", ts1.symbolEquals(ts2));
    }

    public void testTypeNotEqualsDifTypes()
    {
        TypeSymbol ts1 = new TypeSymbol("test1");
        VariableSymbol vs2 = new VariableSymbol("test1", null, null);

        assertFalse("TypeSymbol should be not equal to VariableSymbol", ts1.symbolEquals(vs2));
        assertTrue("VariableSymbol should be not equal to TypeSymbol", vs2.symbolEquals(ts1));
    }

    public void testSymbolEqualsSame()
    {
        VariableSymbol vs1 = new VariableSymbol("test1", null, null);
        VariableSymbol vs2 = new VariableSymbol("test1", null, null);

        assertTrue("VariableSymbols should be equal", vs1.symbolEquals(vs2));
        assertTrue("VariableSymbols should be equal", vs2.symbolEquals(vs1));
        assertTrue("VariableSymbols should be equal", vs1.symbolEquals(vs1));
    }

    public void testSymbolEqualsDiff()
    {
        VariableSymbol vs1 = new VariableSymbol("test1", null, null);
        FieldSymbol fs2 = new FieldSymbol("test1", null, null, null);

        assertTrue("VariableSymbol should be equal to FieldSymbol", vs1.symbolEquals(fs2));
        assertTrue("FieldSymbol should be equal to VariableSymbol", fs2.symbolEquals(vs1));

        //don't need to test for all combinations of symbols since they use the method in Symbol
    }

    public void testSymbolNotEqualsSame()
    {
        VariableSymbol vs1 = new VariableSymbol("test1", null, null);
        VariableSymbol vs2 = new VariableSymbol("test2", null, null);

        assertFalse("VariableSymbols should be not equal", vs1.symbolEquals(vs2));
    }

    public void testSymbolNotEqualsDiff()
    {
        VariableSymbol vs1 = new VariableSymbol("test1", null, null);
        FieldSymbol fs2 = new FieldSymbol("test2", null, null, null);

        assertFalse("VariableSymbol should be not equal to FieldSymbol", vs1.symbolEquals(fs2));
        assertFalse("FieldSymbol should be not equal to VariableSymbol", fs2.symbolEquals(vs1));

        //don't need to test for all combinations of symbols since they use the method in Symbol
    }

    //these are the interesting ones...
    public void testClassEqualsSimple()
    {
        ClassSymbol cs1 = new ClassSymbol("test1", null, null);
        ClassSymbol cs2 = new ClassSymbol("test1", null, null);

        assertTrue("ClassSymbols are equal", cs1.symbolEquals(cs2));
        assertTrue("ClassSymbols are equal", cs2.symbolEquals(cs1));
        assertTrue("ClassSymbols are equal", cs1.symbolEquals(cs1));
    }


    public void testClassNotEqualsSimple()
    {
        ClassSymbol cs1 = new ClassSymbol("test1", null, null);
        ClassSymbol cs2 = new ClassSymbol("test2", null, null);

        assertFalse("ClassSymbols are not equal", cs1.symbolEquals(cs2));
        assertFalse("ClassSymbols are not equal", cs2.symbolEquals(cs1));
    }

    public void testClassInheritEquals()
    {
        ClassSymbol cs1 = new ClassSymbol("test1", null, null);
        ClassSymbol cs2 = new ClassSymbol("test2", cs1, null);
        ClassSymbol cs3 = new ClassSymbol("test3", cs2, null);

        assertFalse("ClassSymbols are not equal", cs2.symbolEquals(cs3));
        assertTrue("ClassSymbols are equal", cs3.symbolEquals(cs2));
    }

    public void testClassSiblingNotEquals()
    {
        ClassSymbol cs1 = new ClassSymbol("test1", null, null);
        ClassSymbol cs2 = new ClassSymbol("test2", cs1, null);
        ClassSymbol cs3 = new ClassSymbol("test3", cs1, null);

        assertFalse("ClassSymbols are not equal", cs2.symbolEquals(cs3));
        assertFalse("ClassSymbols are not equal", cs3.symbolEquals(cs2));
    }
}